@@ -16,27 +16,29 @@
 #
 
 
+from distutils.version import LooseVersion as Version  # pylint: disable=no-name-in-module, import-error
+
 import azurelinuxagent.common.logger as logger
-from azurelinuxagent.common.version import *
-from .default import DefaultOSUtil
+from azurelinuxagent.common.version import DISTRO_NAME, DISTRO_CODE_NAME, DISTRO_VERSION, DISTRO_FULL_NAME
+from .alpine import AlpineOSUtil
 from .arch import ArchUtil
+from .bigip import BigIpOSUtil
 from .clearlinux import ClearLinuxUtil
 from .coreos import CoreOSUtil
 from .debian import DebianOSBaseUtil, DebianOSModernUtil
+from .default import DefaultOSUtil
 from .freebsd import FreeBSDOSUtil
-from .openbsd import OpenBSDOSUtil
-from .redhat import RedhatOSUtil, Redhat6xOSUtil
-from .suse import SUSEOSUtil, SUSE11OSUtil
-from .ubuntu import UbuntuOSUtil, Ubuntu12OSUtil, Ubuntu14OSUtil, \
-    UbuntuSnappyOSUtil, Ubuntu16OSUtil, Ubuntu18OSUtil
-from .alpine import AlpineOSUtil
-from .bigip import BigIpOSUtil
 from .gaia import GaiaOSUtil
 from .iosxe import IosxeOSUtil
+from .mariner import MarinerOSUtil
 from .nsbsd import NSBSDOSUtil
+from .openbsd import OpenBSDOSUtil
 from .openwrt import OpenWRTOSUtil
-
-from distutils.version import LooseVersion as Version
+from .redhat import RedhatOSUtil, Redhat6xOSUtil, RedhatOSModernUtil
+from .suse import SUSEOSUtil, SUSE11OSUtil
+from .photonos import PhotonOSUtil
+from .ubuntu import UbuntuOSUtil, Ubuntu12OSUtil, Ubuntu14OSUtil, \
+    UbuntuSnappyOSUtil, Ubuntu16OSUtil, Ubuntu18OSUtil
 
 
 def get_osutil(distro_name=DISTRO_NAME,
@@ -52,6 +54,9 @@ def get_osutil(distro_name=DISTRO_NAME,
 
 def _get_osutil(distro_name, distro_code_name, distro_version, distro_full_name):
 
+    if distro_name == "photonos":
+        return PhotonOSUtil()
+
     if distro_name == "arch":
         return ArchUtil()
 
@@ -61,16 +66,18 @@ def _get_osutil(distro_name, distro_code_name, distro_version, distro_full_name)
     if distro_name == "ubuntu":
         if Version(distro_version) in [Version("12.04"), Version("12.10")]:
             return Ubuntu12OSUtil()
-        elif Version(distro_version) in [Version("14.04"), Version("14.10")]:
+        if Version(distro_version) in [Version("14.04"), Version("14.10")]:
             return Ubuntu14OSUtil()
-        elif Version(distro_version) in [Version('16.04'), Version('16.10'), Version('17.04')]:
+        if Version(distro_version) in [Version('16.04'), Version('16.10'), Version('17.04')]:
             return Ubuntu16OSUtil()
-        elif Version(distro_version) in [Version('18.04')]:
+        if Version(distro_version) in [Version('18.04'), Version('18.10'),
+                                         Version('19.04'), Version('19.10'),
+                                         Version('20.04')]:
             return Ubuntu18OSUtil()
-        elif distro_full_name == "Snappy Ubuntu Core":
+        if distro_full_name == "Snappy Ubuntu Core":
             return UbuntuSnappyOSUtil()
-        else:
-            return UbuntuOSUtil()
+
+        return UbuntuOSUtil()
 
     if distro_name == "alpine":
         return AlpineOSUtil()
@@ -78,34 +85,38 @@ def _get_osutil(distro_name, distro_code_name, distro_version, distro_full_name)
     if distro_name == "kali":
         return DebianOSBaseUtil()
 
-    if distro_name == "coreos" or distro_code_name == "coreos":
+    if distro_name in ("flatcar", "coreos") or distro_code_name in ("flatcar", "coreos"):
         return CoreOSUtil()
 
-    if distro_name in ("suse", "sles", "opensuse"):
+    if distro_name in ("suse", "sle_hpc", "sles", "opensuse"):
         if distro_full_name == 'SUSE Linux Enterprise Server' \
                 and Version(distro_version) < Version('12') \
                 or distro_full_name == 'openSUSE' and Version(distro_version) < Version('13.2'):
             return SUSE11OSUtil()
-        else:
-            return SUSEOSUtil()
+
+        return SUSEOSUtil()
 
     if distro_name == "debian":
         if "sid" in distro_version or Version(distro_version) > Version("7"):
             return DebianOSModernUtil()
-        else:
-            return DebianOSBaseUtil()
 
-    if distro_name == "redhat" \
-            or distro_name == "centos" \
-            or distro_name == "oracle":
+        return DebianOSBaseUtil()
+
+    if distro_name in ("redhat", "rhel", "centos", "oracle", "almalinux", "cloudlinux"):
         if Version(distro_version) < Version("7"):
             return Redhat6xOSUtil()
-        else:
-            return RedhatOSUtil()
+
+        if Version(distro_version) >= Version("8.6"):
+            return RedhatOSModernUtil()
+
+        return RedhatOSUtil()
 
     if distro_name == "euleros":
         return RedhatOSUtil()
 
+    if distro_name == "uos":
+        return RedhatOSUtil()
+
     if distro_name == "freebsd":
         return FreeBSDOSUtil()
 
@@ -121,14 +132,14 @@ def _get_osutil(distro_name, distro_code_name, distro_version, distro_full_name)
     if distro_name == "iosxe":
         return IosxeOSUtil()
 
+    if distro_name == "mariner":
+        return MarinerOSUtil()
+
     if distro_name == "nsbsd":
         return NSBSDOSUtil()
 
     if distro_name == "openwrt":
         return OpenWRTOSUtil()
 
-    else:
-        logger.warn("Unable to load distro implementation for {0}. Using "
-                    "default distro implementation instead.",
-                    distro_name)
-        return DefaultOSUtil()
+    logger.warn("Unable to load distro implementation for {0}. Using default distro implementation instead.", distro_name)
+    return DefaultOSUtil()
