@@ -109,7 +109,7 @@ class TestImds(AgentTestCase):
         "zone": "In"
         }'''
 
-        data = json.loads(s, encoding='utf-8')
+        data = json.loads(s)
 
         compute_info = imds.ComputeInfo()
         set_properties("compute", compute_info, data)
@@ -258,7 +258,7 @@ class TestImds(AgentTestCase):
             "version": "{3}"
         }}'''.format(publisher, offer, sku, version)
 
-        data = json.loads(s, encoding='utf-8')
+        data = json.loads(s)
         compute_info = imds.ComputeInfo()
         set_properties("compute", compute_info, data)
 
@@ -398,7 +398,7 @@ class TestImds(AgentTestCase):
                 test_subject._http_get = Mock(side_effect=self._mock_http_get)
                 self._mock_imds_setup(primary_ioerror=has_primary_ioerror, secondary_ioerror=True)
                 result = test_subject.get_metadata(resource_path=resource_path, is_health=is_health)
-                self.assertFalse(result.success) if has_primary_ioerror else self.assertTrue(result.success)
+                self.assertFalse(result.success) if has_primary_ioerror else self.assertTrue(result.success)  # pylint: disable=expression-not-assigned
                 self.assertFalse(result.service_error)
                 if has_primary_ioerror:
                     self.assertEqual('IMDS error in /metadata/{0}: Unable to connect to endpoint'.format(resource_path), result.response)
@@ -458,12 +458,12 @@ class TestImds(AgentTestCase):
                 self.assertEqual(2 if has_primary_ioerror else 1, test_subject._http_get.call_count)
 
     def _mock_imds_setup(self, primary_ioerror=False, secondary_ioerror=False, gone_error=False, throttled=False, bad_request=False):
-        self._mock_imds_expect_fallback = primary_ioerror
-        self._mock_imds_primary_ioerror = primary_ioerror
-        self._mock_imds_secondary_ioerror = secondary_ioerror
-        self._mock_imds_gone_error = gone_error
-        self._mock_imds_throttled = throttled
-        self._mock_imds_bad_request = bad_request
+        self._mock_imds_expect_fallback = primary_ioerror  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_primary_ioerror = primary_ioerror  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_secondary_ioerror = secondary_ioerror  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_gone_error = gone_error  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_throttled = throttled  # pylint: disable=attribute-defined-outside-init
+        self._mock_imds_bad_request = bad_request  # pylint: disable=attribute-defined-outside-init
 
     def _mock_http_get(self, *_, **kwargs):
         if "foo.bar" == kwargs['endpoint'] and not self._mock_imds_expect_fallback:
